/*
 * Copyright (C) 2012 Southern Storm Software, Pty Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "DegreeField.h"

/**
 * \class DegreeField DegreeField.h <DegreeField.h>
 * \brief Field that manages the display and editing of a degree,minute second value.
 *
 * DegreeField is suitable for displaying degrees in degrees, minutes, seconds,
 * Degrees are specified
 * in seconds as an <tt>unsigned long</tt> value.  They are displayed
 * as <tt>DDD:MM:SS</tt>, for hours, minutes, and seconds.
 *
 * The time field can be either read-only or read-write.  When read-write,
 * the Up, Down, Left, and Right buttons can be used to modify the hour,
 * minute, and second components of the time value.
 *
 * The following example displays the number of hours, minutes, and seconds
 * since the device was reset, wrapping around after 360 degrees:
 *
 * \code
 * Form mainForm(lcd);
 * DegreeField degreeField(mainForm, "Degrees to turn", 360, TIMEFIELD_READ_ONLY);
 *
 * void loop() {
 *     degreeField.setValue(0);
 *     mainForm.dispatch(lcd.getButton());
 * }
 * \endcode
 *
 * \image html FormTimeRO.png
 *
 * A read-write field can be used to ask the user for the duration of an
 * application count-down timer:
 *
 * \code
 * DegreeField rotationField(mainForm, "Rotation angle", 360, TIMEFIELD_READ_WRITE);
 * \endcode
 *
 * \image html FormTimeRW.png
 *
 * \sa Field
 */

#define EDIT_DEGREE_HUNDRED  0
#define EDIT_DEGREE_TENS    1
#define EDIT_DEGREE         2
#define EDIT_MINUTE_TENS    3
#define EDIT_MINUTE         4
#define EDIT_SECOND_TENS    5
#define EDIT_SECOND         6

/**
 * \brief Constructs a new time field with a specific \a label.
 *
 * The field is initially not associated with a Form.  The field can be
 * added to a form later using Form::addField().
 *
 * Initially value() is 0, maxHours() is 360, and isReadOnly() is
 * DEGREFIELD_READ_WRITE.
 *
 * \sa Form::addField()
 */
DegreeField::DegreeField(const String &label)
    : Field(label)
    , _value(0)
    , _maxDegrees(360)
    , _printLen(0)
    , _readOnly(false)
    , editField(EDIT_DEGREE)
{
}

/**
 * \brief Constructs a new boolean field with a specific \a label and
 * attaches it to a \a form.
 *
 * The initial value() of the field will be 0.  The value() will be limited
 * to be less than \a maxDegrees * 60 * 60 seconds.
 *
 * If \a readOnly is DEGREEFIELD_READ_ONLY, then the field will display degrees
 * but not allow them to be modified by the user.  If \a readOnly is
 * DEGREEFIELD_READ_WRITE, then the field will modifiable by the user.
 *
 * \sa value()
 */
DegreeField::DegreeField(Form &form, const String &label, int maxDegrees, bool readOnly)
    : Field(form, label)
    , _value(0)
    , _maxDegrees(maxDegrees)
    , _printLen(0)
    , _readOnly(readOnly)
    , editField(EDIT_DEGREE)
{
}

int DegreeField::dispatch(int event)
{
    unsigned long newValue;
	#define SECS_PER_DEGREE 3600
    if (_readOnly)
        return -1;
    if (event == LCD_BUTTON_UP) {
        newValue = _value;
        if (editField == EDIT_DEGREE) {
            newValue += SECS_PER_DEGREE;
		} else if (editField == EDIT_DEGREE_HUNDRED) {	// RJW - Need to check validation
					newValue += (unsigned long)(100L * SECS_PER_DEGREE);
	           if(newValue > (unsigned long) (unsigned long)_maxDegrees * (unsigned long) SECS_PER_DEGREE)
					newValue -= (unsigned long)(100L * SECS_PER_DEGREE);
		} else if (editField == EDIT_DEGREE_TENS) {		// RJW - Need to Check validation
              		newValue += (unsigned long)(10L * SECS_PER_DEGREE);
				if(newValue > (unsigned long) (unsigned long)_maxDegrees * (unsigned long) SECS_PER_DEGREE)
					newValue -= (unsigned long)(10L * SECS_PER_DEGREE);
        } else if (editField == EDIT_MINUTE_TENS) {
            if (((newValue / 60) % 60) >= 50)
                newValue -= 50 * 60;
            else
                newValue += 10 * 60;
        } else if (editField == EDIT_MINUTE) {
            if (((newValue / 60) % 60) == 59)
                newValue -= 59 * 60;
            else
                newValue += 60;
        } else if (editField == EDIT_SECOND_TENS) {
            if ((newValue % 60) >= 50)
                newValue -= 50;
            else
                newValue += 10;
        } else {
            if ((newValue % 60) == 59)
                newValue -= 59;
            else
                newValue += 1;
        }
        setValue(newValue);
        return FORM_CHANGED;
    } else if (event == LCD_BUTTON_DOWN) {
        newValue = _value;
        if (editField == EDIT_DEGREE) {
            if (newValue < 60 * 60)
                newValue += ((unsigned long)(_maxDegrees - 1)) * 60 * 60;
            else
                newValue -= 60 * 60;
		} else if (editField == EDIT_DEGREE_TENS) {		// RJW - Need to Check validation
            if(newValue > (unsigned long)(10L * SECS_PER_DEGREE))
				newValue -= (unsigned long)(10L * SECS_PER_DEGREE);
		} else if (editField == EDIT_DEGREE_HUNDRED) {	// RJW - Need to check validation
          if(newValue > (unsigned long)(100L * SECS_PER_DEGREE))
				newValue -= (unsigned long)(100L * SECS_PER_DEGREE);
        } else if (editField == EDIT_MINUTE_TENS) {
            if (((newValue / 60) % 60) < 10)
                newValue += 50 * 60;
            else
                newValue -= 10 * 60;
        } else if (editField == EDIT_MINUTE) {
            if (((newValue / 60) % 60) == 0)
                newValue += 59 * 60;
            else
                newValue -= 60;
        } else if (editField == EDIT_SECOND_TENS) {
            if ((newValue % 60) < 10)
                newValue += 50;
            else
                newValue -= 10;
        } else {
            if ((newValue % 60) == 0)
                newValue += 59;
            else
                newValue -= 1;
        }
        setValue(newValue);
        return FORM_CHANGED;
    } else if (event == LCD_BUTTON_LEFT) {
        if (editField != EDIT_DEGREE_HUNDRED) {
            --editField;
            printDegrees();
            return 0;
        }
    } else if (event == LCD_BUTTON_RIGHT) {
        if (editField != EDIT_SECOND) {
            ++editField;
            printDegrees();
            return 0;
        }
    }
    return -1;
}

void DegreeField::enterField(bool reverse)
{
    Field::enterField(reverse);
    if (reverse)
        editField = EDIT_SECOND;
    else
        editField = EDIT_DEGREE;
    printDegrees();
    if (!_readOnly)
        lcd()->cursor();
}

void DegreeField::exitField()
{
    if (!_readOnly)
        lcd()->noCursor();
    Field::exitField();
}

/**
 * \fn unsigned long TimeField::value() const
 * \brief Returns the current value of this time field, in seconds.
 *
 * \sa setValue()
 */

/**
 * \brief Sets the \a value of this time field, in seconds.
 *
 * If \a value is greater than or equal to maxHours() * 60 * 60,
 * then it will be wrapped around to fall within the valid range.
 *
 * \sa value(), maxDegrees()
 */
void DegreeField::setValue(unsigned long value)
{
    unsigned long maxSecs = ((unsigned long)_maxDegrees) * 60 * 60;
    value %= maxSecs;
    if (value != _value) {
        _value = value;
        if (isCurrent())
            printDegrees();
    }
}

/**
 * \fn int DegreeField::maxHours() const
 * \brief Returns the maximum number of hours before the field wraps around.
 *
 * \sa setMaxDegrees(), setValue()
 */

/**
 * \fn void DegreeField::setMaxDegrees(int maxDegrees)
 * \brief Sets the maximum number of degrees before the field wraps around
 * to \a maxDegrees.
 *
 * \sa maxDegrees(), setValue()
 */

/**
 * \fn bool DegreeField::readOnly() const
 * \brief Returns DEGREEFIELD_READ_ONLY (true) or DEGREEFIELD_READ_WRITE (false).
 *
 * \sa setReadOnly()
 */

/**
 * \brief Sets the read-only state of this field to \a value.
 *
 * The \a value should be one of DEGREEFIELD_READ_ONLY (true) or
 * DEGREEFIELD_READ_WRITE (false).  Use of the named constants is recommended.
 *
 * \sa readOnly()
 */
void DegreeField::setReadOnly(bool value)
{
    if (_readOnly != value) {
        _readOnly = value;
        printDegrees();
        if (isCurrent()) {
            if (value)
                lcd()->cursor();
            else
                lcd()->noCursor();
        }
    }
}

void DegreeField::printDegrees()
{
    lcd()->setCursor(0, 1);
    int col = printField(_value / (60 * 60), 1);
    int degreeCol = col - 1;
	lcd()->write(223);
    ++col;
    col += printField((_value / 60) % 60,0);
    int minuteCol = col - 1;
	lcd()->write(39);
    ++col;
    col += printField(_value % 60,0);
    int secondCol = col - 1;
    int tempCol = col;
    while (tempCol++ < _printLen)
        lcd()->write(' ');
    _printLen = col;
	lcd()->write(34);

    if (!_readOnly) {
        if (editField == EDIT_DEGREE_HUNDRED){
            lcd()->setCursor(degreeCol - 2, 1);
		}
        else if (editField == EDIT_DEGREE_TENS)
            lcd()->setCursor(degreeCol - 1, 1);
        else if (editField == EDIT_DEGREE)
            lcd()->setCursor(degreeCol, 1);		
        else if (editField == EDIT_MINUTE_TENS)
            lcd()->setCursor(minuteCol - 1, 1);
        else if (editField == EDIT_MINUTE)
            lcd()->setCursor(minuteCol, 1);
        else if (editField == EDIT_SECOND_TENS)
            lcd()->setCursor(secondCol - 1, 1);
        else
            lcd()->setCursor(secondCol, 1);
    }
}

int DegreeField::printField(unsigned long value, int degree)
{
	int retval = 2;
    if (value < 100) {
		if (degree){
			lcd()->write('0');	
			retval++;
		}		
        lcd()->write('0' + (int)(value / 10));
        lcd()->write('0' + (int)(value % 10));
        return retval;
    }
    unsigned long divisor = 100;
    while ((value / divisor) >= 10)
        divisor *= 10;
    int digits = 0;
    while (divisor > 0) {
        lcd()->write('0' + (int)((value / divisor) % 10));
        divisor /= 10;
        ++digits;
    }
    return digits;
}
