/*
 * Copyright (C) 2012 Southern Storm Software, Pty Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "GetIntField.h"

/**
 * \class  GetIntField.h <GetIntField.h>
 * \brief Field that manages the display and editing of a Inttger value.
 *
 * GetIntField is suitable for displaying and editing int values
 *
 * The int field can be either read-only or read-write.  When read-write,
 * the Up, Down, Left, and Right buttons can be used to modify the
 * components of the Integer value.
 *
 * The following example displays the number
 *
 * \code
 * Form mainForm(lcd);
 * GetIntField intField(mainForm, "Number of steps", 9999, GETINTFIELD_READ_WRITE);
 *
 * void loop() {
 *     degreeField.setValue(millis() / 1000);
 *     mainForm.dispatch(lcd.getButton());
 * }
 * \endcode
 *
 * \image html FormTimeRO.png
 *
 *
 * \image html FormTimeRW.png
 *
 * \sa Field
 */
      

#define EDIT_INT_MIL     0  
#define EDIT_INT_HTHOU   1  
#define EDIT_INT_TTHOU   2  
#define EDIT_INT_THOU    3  
#define EDIT_INT_HUND    4
#define EDIT_INT_TENS    5
#define EDIT_INT         6

/**
 * \brief Constructs a new time field with a specific \a label.
 *
 * The field is initially not associated with a Form.  The field can be
 * added to a form later using Form::addField().
 *
 * Initially value() is 0, maxHours() is 360, and isReadOnly() is
 * DEGREFIELD_READ_WRITE.
 *
 * \sa Form::addField()
 */
GetIntField::GetIntField(const String &label)
    : Field(label)
    , _value(0)
    , _maxIntField(9999)
    , _printLen(0)
    , _readOnly(false)
    , editField(EDIT_INT)
{
	editField = EDIT_INT;
}

/**
 * \brief Constructs a new boolean field with a specific \a label and
 * attaches it to a \a form.
 *
 * The initial value() of the field will be 0.  The value() will be limited
 * to be less than \a maxIntField.
 *
 * If \a readOnly is GETINTFIELD_READ_ONLY, then the field will display the number
 * but not allow it to be modified by the user.  If \a readOnly is
 * GETINTFIELD_READ_WRITE, then the field will modifiable by the user.
 *
 * \sa value()
 */
GetIntField::GetIntField(Form &form, const String &label, unsigned long maxIntField, bool readOnly)
    : Field(form, label)
    , _value(0)
    , _maxIntField(maxIntField)
    , _printLen(0)
    , _readOnly(readOnly)
    , editField(EDIT_INT)
{
	setMaxGetIntField(maxIntField);
}

int GetIntField::dispatch(int event)
{
    unsigned long newValue;
    if (_readOnly)
        return -1;
	lcd()->setCursor(editField, 1);
    if (event == LCD_BUTTON_UP) {
        newValue = _value;
        if (editField == EDIT_INT) {
			if(_value < maxGetIntField())
	            newValue += 1L;
        } else if (editField == EDIT_INT_TENS) {
			if(_value <= (maxGetIntField() - (unsigned long)10L))
	            newValue += 10L;
        } 
		else if (editField == EDIT_INT_HUND) {
			if(maxGetIntField()>=(unsigned long) 100L){
				if (_value <= maxGetIntField() - (unsigned long) 100L){
				 	newValue += (unsigned long)100L;
				}
			}
        }
		 else if (editField == EDIT_INT_THOU) {
			if(maxGetIntField()>=(unsigned long) 1000L){
				if (_value <= maxGetIntField() - (unsigned long) 1000L){
				 	newValue += (unsigned long)1000L;
				}
			}
        }
		 else if (editField == EDIT_INT_TTHOU) {
			if(maxGetIntField()>=(unsigned long) 10000L){
				if (_value <= maxGetIntField() - (unsigned long) 10000L){
				 	newValue += (unsigned long)10000L;
				}
			}
        }
		 else if (editField == EDIT_INT_HTHOU) {
			if(maxGetIntField()>=(unsigned long) 100000L){
				if (_value <= maxGetIntField() - (unsigned long) 100000L){
				 	newValue += (unsigned long)100000L;
				}
			}
        }
		 else {					// EDIT_INT_MIL
			if(maxGetIntField()>=(unsigned long) 1000000L){
				if (_value <= maxGetIntField() - (unsigned long) 1000000L){
				 	newValue += (unsigned long)1000000L;
				}
			}
        }
        setValue(newValue);
   	    printValue(_value);
		lcd()->setCursor(editField, 1);
	    return FORM_CHANGED;
    } else if (event == LCD_BUTTON_DOWN) {
        newValue = _value;
        if (editField == EDIT_INT) {		
			if(newValue > 0)
	                  newValue -= 1;
        } else if (editField == EDIT_INT_TENS) { 
			if(_maxIntField >= 10L && newValue >= 10L)
	                  newValue -= 10L;
        } else if (editField == EDIT_INT_HUND) {
			if(_maxIntField >= 100L && newValue >= 100L)
	                  newValue -= 100L;
        } else if (editField == EDIT_INT_THOU) {
			if(_maxIntField >= 1000L && newValue >= 1000L)
	                  newValue -= 1000L;
        } else if (editField == EDIT_INT_TTHOU) {
			if(_maxIntField >= 10000L && newValue >= 10000L)
	                  newValue -= 10000L;
        } else if (editField == EDIT_INT_HTHOU) {
			if(_maxIntField >= 100000L && newValue >= 100000L)
	                  newValue -= 100000L;
        } else {					// EDIT_INT_MIL
			if((_maxIntField >= 1000000L) && (newValue >= 1000000L))
	                  newValue -= 1000000L;
        }
        setValue(newValue);
	    printValue(_value);
		lcd()->setCursor(editField, 1);
        return FORM_CHANGED;
    } else if (event == LCD_BUTTON_LEFT) {
        if (editField != EDIT_INT_MIL) {
            --editField;
			lcd()->setCursor(editField, 1);
            return 0;
        }
    } else if (event == LCD_BUTTON_RIGHT) {
        if (editField != EDIT_INT) {
            ++editField;
			lcd()->setCursor(editField+1, 1);          
            return 0;
        }
    } 
    return -1;
}

void GetIntField::enterField(bool reverse)
{
    Field::enterField(reverse);
	printValue(_value);
    if (!_readOnly)
        lcd()->cursor();
}

void GetIntField::exitField()
{
    if (!_readOnly)
        lcd()->noCursor();
    Field::exitField();
}

/**
 * \fn unsigned long TimeField::value() const
 * \brief Returns the current value of this time field, in seconds.
 *
 * \sa setValue()
 */

/**
 * \brief Sets the \a value of this time field, in seconds.
 *
 * If \a value is greater than or equal to maxHours() * 60 * 60,
 * then it will be wrapped around to fall within the valid range.
 *
 * \sa value(), maxDegrees()
 */
void GetIntField::setValue(unsigned long value)
{
    unsigned long maxSecs = ((unsigned long)_maxIntField) * 60 * 60;
    value %= maxSecs;
    if (value != _value) {
        _value = value;
        if (isCurrent())
            printValue(_value);
    }
}

/**
 * \brief Sets the read-only state of this field to \a value.
 *
 * The \a value should be one of DEGREEFIELD_READ_ONLY (true) or
 * DEGREEFIELD_READ_WRITE (false).  Use of the named constants is recommended.
 *
 * \sa readOnly()
 */
void GetIntField::setReadOnly(bool value)
{
    if (_readOnly != value) {
        _readOnly = value;
		printValue(_value);
        if (isCurrent()) {
            if (value)
                lcd()->cursor();
            else
                lcd()->noCursor();
        }
    }
}


void GetIntField::printValue(unsigned long value)
{
    String str(_value);
	_printLen = str.length();
    lcd()->setCursor(0, 1);
    unsigned int len = _printLen;
    while (len++ < EDIT_INT+1)
        lcd()->write(' ');
    lcd()->print(str);
	lcd()->setCursor(EDIT_INT, 1);
}
